import glob
import json
import os
from pathlib import Path

import axipy
from axipy import *
from PySide2.QtCore import Qt
from PySide2.QtGui import QIcon
from PySide2.QtWidgets import QDialog, QToolButton, QListWidgetItem, QWidget, QCheckBox, QLabel, QHBoxLayout, \
    QAbstractItemView, QFileDialog
#from axipy import Notifications

from CatalogPointsNew.ui.DlgAngle import DlgAngle
#from QtWidgets import QListWidgetItem, QWidget, QCheckBox, QLabel, QHBoxLayout

from CatalogPointsNew.ui.DlgCoords import DlgCoords
from CatalogPointsNew.ui.DlgDistance import DlgDistance
from CatalogPointsNew.ui.DlgIdPoints import DlgIdPoints
from CatalogPointsNew.ui.DlgIdReference import DlgReferenceField
from CatalogPointsNew.ui.dlgrunner import DlgRunPrigress
from CatalogPointsNew.ui.tools.buildCatalogPoints import BuilderCatalogPoints
from CatalogPointsNew.ui.utils.ui_tools import load_ui, textToBool


class DlgCatalogPoints(QDialog):
    def __init__(self,json_params=None,parent=None):
        super(DlgCatalogPoints, self).__init__(parent)
        self.__def_params=json_params
        self.__out_ext = '.tab'
        self.__all_widget=None
        path_ui=os.path.join(os.path.dirname(__file__), "DialogListProperty_1.ui")
        self.window = load_ui(path_ui,parent)
        self.window.setWindowFlag(Qt.WindowContextHelpButtonHint, False)
        curent_path = str(Path(__file__).parent)
        self.__path_icon = os.path.join(curent_path, "icons")
        #path_add=os.path.join(self.__path_icon, "add.png")
        self.window.pb_load.setIcon(QIcon( os.path.join(self.__path_icon, "open-32.png")))
        self.window.pb_save_as.setIcon(QIcon(os.path.join(self.__path_icon, "saveas.png")))
        self.window.tlb_saveas_cat.setIcon(QIcon(os.path.join(self.__path_icon, "saveas.png")))
        #self.window.tb_item_move_down.setIcon(QIcon(os.path.join(self.__path_icon, "down.png")))
        #self.window.tb_item_move_up.setIcon(QIcon(os.path.join(self.__path_icon, "up.png")))
        if self.__def_params is None:
            ''' Загружаем последний файл config'''
            self.__loadConfig()
        if self.__all_widget is None:
            self.__init_childrenWdget()
        self.__initView()
        #self.window.list_out_items.setDragDropMode(QAbstractItemView.DragDrop)
        #self.window.list_out_items.currentItemChanged.connect(self.__change_select_item)
        self.window.cmb_tables.currentIndexChanged.connect(self.__change_table_source)
        self.window.pb_load.clicked.connect(self.__loadFileConfig)
        self.window.pb_save_as.clicked.connect(self.__saveFileConfig)
        self.window.tlb_saveas_cat.clicked.connect(self.__saveCatalog)
        self.window.pb_run.clicked.connect(self.__run)
        self.window.pb_cancel.clicked.connect(self.__close)
        self.window.list_out_items.setCurrentRow(0)

        #self.window.tb_item_move_down.clicked.connect(self.__item_down)
        #self.window.tb_item_move_up.clicked.connect(self.__item_up)
        #self.window.tb_del.clicked.connect(self.__disable_item)
    def __initView(self):
        if self.__all_widget is None:
            return
        for w_curent in self.__all_widget:
            self.__addWidget(w_curent['widget'],w_curent['active'])
        return
        #w_idPoint = DlgIdPoints(self.window)
        #self.__addWidget(w_idPoint)
        #self.__w_coord=DlgCoords(self.window)
        #self.__addWidget(self.__w_coord)
    def __init_childrenWdget(self):
        list_widgets=self.__def_params['template_params']['widget']
        self.__all_widget=[]
        for item in list_widgets:
            name=item['name']
            if name.lower()=='idreference':
                isActive=textToBool(item['active'])
                self.__all_widget.append({'name':'idReference','widget': DlgReferenceField(item['params'],self.window),'active':isActive,"clsname":item['name_cls']})
                continue
            if name.lower()=='idpoints':
                isActive=textToBool(item['active'])
                self.__all_widget.append({'name':'idPoints','widget': DlgIdPoints(item['params'],self.window),'active':isActive,"clsname":item['name_cls']})
                continue
            if name.lower()=='coords_points':
                isActive=textToBool(item['active'])
                self.__all_widget.append({'name':'coords_points','widget': DlgCoords(item['params'],self.window),'active':isActive,"clsname":item['name_cls']})
                continue
            if name.lower()=='distance':
                isActive=textToBool(item['active'])
                self.__all_widget.append({'name':'distance','widget': DlgDistance(item['params'],self.window),'active':isActive,"clsname":item['name_cls']})
                continue
            if name.lower()=='angle':
                isActive=textToBool(item['active'])
                self.__all_widget.append({'name':'angle','widget': DlgAngle(item['params'],self.window),'active':isActive,"clsname":item['name_cls']})
                continue

    def __addWidget(self,widget,isActive=True):

        item, item_widget=self.__createItem(widget,isActive)
        self.window.list_out_items.addItem(item)
        self.window.list_out_items.setItemWidget(item, item_widget)
    def __disable_item(self):
        curent_item= self.window.list_out_items.currentItem()
        #self.window.list_out_items.currentItem().setFlags(Qt.NoItemFlags)
    def __loadConfig(self,name_file=None):

        if name_file is None:
            path_config=str(Path(__file__).parent.joinpath("config_files","*.json"))
            #os.path.join(pathfile, "output", "log.txt").
            files = glob.glob(path_config)
            files.sort(key=os.path.getmtime)
            last_config_file=files[-1]
        else:

            last_config_file=name_file
        self.__def_params = None
        with open(last_config_file, 'r', encoding='utf-8') as fcc_file:
            self.__def_params = json.load(fcc_file)
        self.window.label_name_file_conf.setText(str(Path(last_config_file).name))
        if name_file is not None:
            self.window.list_out_items.clear()
            self.__init_childrenWdget()
            self.__initView()
    def __item_up(self):
        currentRow = self.window.list_out_items.currentRow()
        currentItem = self.window.list_out_items.takeItem(currentRow)
        self.window.list_out_items.insertItem(currentRow - 1, currentItem)
    def __item_down(self):
        currentRow = self.window.list_out_items.currentRow()
        currentItem = self.window.list_out_items.takeItem(currentRow)
        #nextItem = self.window.list_out_items.takeItem(currentRow+1)
        self.window.list_out_items.insertItem(currentRow + 1, currentItem)
        #self.window.list_out_items.insertItem(currentRow, nextItem)
        jkl=0
    def __change_select_item(self):
        index_select_item=self.window.list_out_items.currentRow()
        if index_select_item==0:
            self.window.tb_item_move_up.setEnabled(False)
        else:
            self.window.tb_item_move_up.setEnabled(True)
        if index_select_item==(self.window.list_out_items.count()-1):
            self.window.tb_item_move_down.setEnabled(False)
        else:
            self.window.tb_item_move_down.setEnabled(True)

    def __change_table_source(self,index):
        tab_select=axipy.da.data_manager.find(self.window.cmb_tables.currentText ())
        path_source=None
        try:
            path_source=tab_select.properties['tabFile']
        except:
            path_source=tab_select.properties['fileName']
        path_src=Path(path_source)
        path_dest=path_src.parent.joinpath(path_src.stem+"_catalog.tab")
        self.window.ln_out_file.setText(str(path_dest))
        self.window.ln_out_file.setCursorPosition(0)
        for widgets in self.__all_widget:
            try:
                widgets['widget'].setCoordSystem(tab_select.coordsystem)
            except:
                pass
            try:
                widgets['widget'].setCurentTable(tab_select)
            except:
                pass
    def clicked(self):
        sender = self.sender()
        push_button = self.window.findChild(QToolButton, sender.objectName())
        ''' Поиск контрола в котором нажали кнопку'''
        print(f'click: {push_button.objectName()}')
        ''' Поиск контрола в котором нажали кнопку'''
        for w_curent in self.__all_widget:
            if push_button.objectName()==w_curent['widget'].Name:
                #w_curent['widget'].window.exec_()
                w_curent['widget'].show()
                return
    def __loadFileConfig(self):
        path_config=str(Path(__file__).parent.joinpath("config_files"))
        files = QFileDialog.getOpenFileName(self.window, 'Загрузить файл настроек', path_config, " (*.json)")
        if not(files[0]==''):
            self.__loadConfig(files[0])
    def __saveFileConfig(self):
        path_config = str(Path(__file__).parent.joinpath("config_files"))
        filename = QFileDialog.getSaveFileName(self.window, 'Сохранить файл настроек', path_config, " (*.json)")
        if filename[0]=='':
            return

        count_row=self.window.list_out_items.count()
        params_widget=[]
        #{'template_params':[]}
        for i in range(count_row):
            item_widget=self.window.list_out_items.item(i)
            name=item_widget.whatsThis()
            widget_l = list(filter(lambda widgets: widgets['widget'].Name == name, self.__all_widget))
            widget=widget_l[0]
            chek_box= self.window.findChild(QCheckBox, name)
            isActive=chek_box.isChecked()
            strIsActive="False"
            if isActive:
                strIsActive = "True"
            widget_params={}
            widget_params['active']=strIsActive
            widget_params['name']=widget['name']
            widget_params['name_cls']=widget['clsname']
            widget_params['params']=widget['widget'].params
            params_widget.append(widget_params)
        all_widgets={'widget':params_widget}
        params={'template_params':all_widgets}
        with open(filename[0], "w", encoding='utf-8') as jsonfile:
            json.dump(params, jsonfile, ensure_ascii=False)
    def __getParams(self):
        count_row = self.window.list_out_items.count()
        params_widget = []
        # {'template_params':[]}
        for i in range(count_row):
            item_widget = self.window.list_out_items.item(i)
            name = item_widget.whatsThis()
            widget_l = list(filter(lambda widgets: widgets['widget'].Name == name, self.__all_widget))
            widget = widget_l[0]
            chek_box = self.window.findChild(QCheckBox, name)
            isActive = chek_box.isChecked()
            strIsActive = "False"
            if isActive:
                strIsActive = "True"
            widget_params = {}
            widget_params['active'] = strIsActive
            widget_params['name'] = widget['name']
            widget_params['name_cls'] = widget['clsname']
            widget_params['params'] = widget['widget'].params
            params_widget.append(widget_params)
        all_widgets = {'widget': params_widget}
        params = {'template_params': all_widgets}
        return params
    def __saveCatalog(self):
        filename = QFileDialog.getSaveFileName(self.window, 'Выбрать файл для сохранения каталога', self.window.ln_out_file.text(), "MapInfo (*.tab);;Excel (*.xlsx)")
        if filename[0]=='':
            return
        self.window.ln_out_file.setText(filename[0])
        self.window.ln_out_file.setCursorPosition(0)
        self.__out_ext=filename[1].lower()
    def __initListTable(self):
        self.window.cmb_tables.clear()
        tables=axipy.da.data_manager.tables
        if axipy.da.data_manager.selection is not None:
            self.window.cmb_tables.addItem(axipy.da.data_manager.selection.name)
        for tab in tables:
            if tab.name=='Выборка':
                continue
            self.window.cmb_tables.addItem(tab.name)
        self.window.cmb_tables.setCurrentIndex (0)
    def __createItem(self,widget,isActive=True):
        item = QListWidgetItem()
        item_widget = QWidget()
        line_chekBox = QCheckBox()
        if isActive:
            line_chekBox.setChecked(True)
        line_chekBox.setObjectName(widget.Name)
        # line_chekBox.setCheckState(True)
        line_text = QLabel(widget.Name)
        # if not widget.isRequired:
        line_push_button = QToolButton()

        path_add_png = os.path.join(self.__path_icon, "config4.png")

        line_push_button.setIcon(QIcon(path_add_png))
        line_push_button.setObjectName(widget.Name)
        line_push_button.clicked.connect(self.clicked)
        item_layout = QHBoxLayout()
        item_layout.addWidget(line_chekBox)
        item_layout.addWidget(line_text)
        item_layout.addWidget(line_push_button)
        item_widget.setLayout(item_layout)
        item.setSizeHint(item_widget.sizeHint())
        item.setWhatsThis(widget.Name)
        return item,item_widget


    def __run_old(self):
        params={}
        params['source_tab'] = self.window.cmb_tables.currentText()
        params['catalog_param'] = self.__getParams()
        params['out'] = self.window.ln_out_file.text()
        if self.__out_ext=='.tab':
            ''' каталог в tab файл '''
            name_tab=str(Path(params['out']).stem)
            tab=axipy.da.data_manager.find(name_tab)
            if tab is not None:
                Notifications.push('Предупреждение', 'Таблица с именем '+name_tab+' будет закрыта', Notifications.Warning)
                tab.close()
        self.window.pb_run.setEnabled(False)
        self.window.pb_cancel.setEnabled(False)
        csl_catalog = BuilderCatalogPoints(None)
        csl_catalog.setParams(params)
        csl_catalog.run()
        self.window.pb_run.setEnabled(True)
        self.window.pb_cancel.setEnabled(True)
        Notifications.push('Создание каталога завершено', 'Каталог в файле '+params['out'], Notifications.Success)
        self.window.close()
    def __run(self):
        params={}
        params['source_tab'] = self.window.cmb_tables.currentText()
        params['catalog_param'] = self.__getParams()
        params['out'] = self.window.ln_out_file.text()
        if self.__out_ext=='.tab':
            ''' каталог в tab файл '''
            name_tab=str(Path(params['out']).stem)
            tab=axipy.da.data_manager.find(name_tab)
            if tab is not None:
                Notifications.push('Предупреждение', 'Таблица с именем '+name_tab+' будет закрыта', Notifications.Warning)
                tab.close()
        self.window.pb_run.setEnabled(False)
        self.window.pb_cancel.setEnabled(False)
        #csl_catalog = BuilderCatalogPoints(None)
        #csl_catalog.setParams(params)
        #csl_catalog.run()
        #try:
        cls_prunProgress=DlgRunPrigress("Создание каталога точек",self.window)
            #cls_prunProgress.exec_(params)
        cls_prunProgress.show(params)
        #except Exception as ex:
        #    pass
        self.window.pb_cancel.setEnabled(True)
        self.window.pb_run.setEnabled(True)


        #Notifications.push('Создание каталога завершено', 'Каталог в файле '+params['out'], Notifications.Success)
        name_tab = str(Path(params['out']).stem)
        tab = axipy.da.data_manager.find(name_tab)
        if tab is not None:
            layer = Layer.create(tab)
            mainwindow.add_layer_interactive(layer)
        self.window.close()
    def __close(self):
        self.window.close()
    def show(self):
        self.__initListTable()

        self.window.show()