from pathlib import Path

import axipy

from PySide2.QtCore import QObject, Signal

from CatalogPointsNew.ui.tools.recordCatalog import CatalogRecords
from CatalogPointsNew.ui.tools.toolOut import writeXlsx, writeToTab
def findInListDict(list_dict,name_field,value):
    for item_dict in list_dict:
        try:
            if item_dict[name_field]==value:
                return item_dict[name_field]
        except:
            pass
    return None
def findPoints(pnt,nameId,nameX,nameY,cat_points):
    xs = list(filter(lambda point: point['name'] == nameX, pnt))
    ys = list(filter(lambda point: point['name'] == nameY, pnt))
    x=xs[0]['value']
    y=ys[0]['value']
    count=len(cat_points[nameX])
    for i in range(count):
        cur_x=cat_points[nameX][i]
        cur_y = cat_points[nameY][i]
        if cur_x==x and cur_y==y:
            id_point=cat_points[nameId][i]
            return id_point
    return None
class BuilderCatalogPoints(QObject):
    updateProcess = Signal(float)
    updateFormatProgressBar = Signal(str)
    finished = Signal(int)

    def __init__(self, parent_cls):
        super(BuilderCatalogPoints, self).__init__()
        self.__parent = parent_cls
        self.__old_val_progress = 0
        self.__cancelProcess = False
    def setParams(self,params):
        self.__params=params
    def __process_callback(self, complete):
        # print('progress: {}, message: "{}", unknown {}'.format(complete, message,user_data))
        if self.__cancelProcess:
            return 0
        cur_progress_int=int(complete*100+0.5)
        if cur_progress_int>self.__old_val_progress:
            self.updateProcess.emit(complete * 100)
            self.__old_val_progress=cur_progress_int
        else:
            if cur_progress_int<self.__old_val_progress:
                self.__old_val_progress = cur_progress_int
                self.updateProcess.emit(complete * 100)
        #time.sleep(0.001)
        # self.__parent.window.pr_bar.setValue(complete*100)
        # print(complete*100)
        return 1
    def __isCancel(self):
        self.__cancelProcess=True

    def setCancel(self):
        self.__cancelProcess = True
    def run(self):
        source_table_name=self.__params['source_tab']
        param_catalog=self.__params['catalog_param']
        table_source=axipy.data_manager.find(source_table_name)
        path_out=self.__params['out']
        try:
            self.__parent.disconnect(self.__isCancel)
        except:
            pass
        self.__parent.cancelProcess.connect(self.__isCancel)
        self.__cancelProcess = False
        self.updateFormatProgressBar.emit("Каталог точек .. %p%")
        self.updateProcess.emit(0)
        self.__cat_points=CatalogRecords(param_catalog,table_source)
        struct_catalog=self.__cat_points.structura
        self.__cat_points.start()
        self_out= {}
        for item_struct in struct_catalog:
            self_out[item_struct['name']]=[]
        self_out['geometry$$$']=[]
        list_name_item= list(self_out.keys())
        ft_count=table_source.count()
        for i,ft in enumerate(table_source.items()):
            is_cancel = self.__process_callback(i / ft_count)
            if is_cancel == 0:
                break
            if ft.geometry is None:
                continue
            points_ft=self.__cat_points.catalogPoints(ft)
            if points_ft is None:
                continue
            is_topology,name_id,nameX,nameY=self.__cat_points.IsTopology

            for pt_ft in points_ft:
                if is_topology and (nameX is not None) and (nameY is not None):
                    ''' поиск точки в каталоге'''

                    if len(self_out[nameX])>0:
                        id_point=findPoints(pt_ft,name_id,nameX,nameY,self_out)
                        if id_point is not None:
                            ids = list(filter(lambda point: point['name'] == name_id, pt_ft))
                            ids[0]['value']=id_point
                for name in list_name_item:
                    try:
                        value_field=list(filter(lambda row: row['name'] == name, pt_ft))
                        #value_field=findInListDict(pt_ft,'name',name)
                        if len(value_field)==0:
                        #if value_field is  None:
                            self_out[name].append(None)
                        else:
                            self_out[name].append(value_field[0]['value'])
                        jkl=0
                        #self_out[name].append(pt_ft['value'])
                    except:
                        self_out[name].append(None)

                ''' еще проверяем на наличие геометрии 
                try:
                    #value_field_geometry = pt_ft['geometry$$$']
                    #value_field_geometry = findInListDict(pt_ft, 'name', 'geometry$$$')
                    value_field_geometry = list(filter(lambda row: row['name'] =='geometry$$$', pt_ft))
                    if len(value_field_geometry)>0:
                        #pnt_value=value_field_geometry[0]['value']
                        self_out['geometry$$$'].append(value_field_geometry[0]['value'])
                except:
                    self_out['geometry$$$'].append(None)
                '''
            #self_out.append(points_ft)
            jkl=0
        type_out=str(Path(path_out).suffix).lower().replace('.','')
        self.updateFormatProgressBar.emit("Сохранение каталога .. %p%")
        if type_out=='xlsx':
            writeXlsx(path_out,self_out)
        if type_out=='tab':
            writeToTab(path_out,struct_catalog,self_out,table_source.coordsystem)
        self.updateProcess.emit(100)
        self.finished.emit(100)