from pathlib import Path

import xlsxwriter
from axipy import Attribute, Schema, provider_manager, Feature, Point, Style

from CatalogPointsNew.ui.utils.help_table import TableCache

def find_new_name(path_tab):
    index=0
    path=Path(path_tab)
    name=str(path.stem)
    for i in range(100):
        new_path=path.parent.joinpath(name+"_"+str(index)+".tab")
        if not new_path.exists():
            return str(new_path)
    return path.parent.joinpath(name+"_"+str(1000)+".tab")


def writeXlsx(path_xlsx,data_out,name_worksheet='Каталог точек',start_col=0,start_row=0):
    workbook = xlsxwriter.Workbook(path_xlsx)
    worksheet = workbook.add_worksheet(name_worksheet)
    list_name_item = list(data_out.keys())
    cur_row=start_row
    cur_col=start_col
    ''' Заголовок'''
    for name in list_name_item:
        if name=='geometry$$$':
            continue
        worksheet.write(cur_row, cur_col , name)
        cur_col=cur_col+1
    cur_row=cur_row+1
    cont_row=len(data_out[list_name_item[0]])
    for i in range(cont_row):
        cur_col = start_col
        for name in list_name_item:
            if name == 'geometry$$$':
                continue
            value=data_out[name][i]
            if value is not None:
                worksheet.write(cur_row, cur_col, value)
            cur_col = cur_col + 1
        cur_row = cur_row + 1
    workbook.close()

def writeToTab(path_out,structure,data_out,cs_out=None):
    def createAttr(structure):
        attr=[]
        for item in structure:
            if item['type']=='int':
                attr.append(Attribute.integer(item['name']))
                continue
            if item['type']=='float':
                attr.append(Attribute.float(item['name']))
                continue
            if item['type']=='str' or item['type']=='string':
                attr.append(Attribute.string(item['name'],item['size']))
                continue
        return attr

    attrs=createAttr(structure)
    shema_tab= Schema(*attrs, coordsystem=cs_out)
    definition = {'src': path_out,
                  'schema': shema_tab,
                  'hidden': False
                  }
    #table_dest=provider_manager.create_open(path_out,definition)
    temp_path_out=path_out
    if Path(path_out).exists():
        temp_path_out=find_new_name(path_out)

    table_dest=provider_manager.tab.create_open(temp_path_out,shema_tab)
    list_name_item = list(data_out.keys())
    cont_row = len(data_out[list_name_item[0]])
    def_style_point=Style.from_mapinfo('Symbol (34, 0, 4)')
    tab_cach=TableCache(table_dest)
    for i in range(cont_row):
        att_value={}
        for name in list_name_item:
            geo_point=None

            value=data_out[name][i]
            if value is not None:
                if name == 'geometry$$$' and cs_out is not None:
                    geo_point = Point(value.x,value.y,cs=cs_out)
                else:
                    att_value[name]=value
        if geo_point is not None:
            new_ft=Feature(att_value,geometry=geo_point,style=def_style_point)
        else:
            new_ft = Feature(att_value)
        #table_dest.insert(new_ft)
        tab_cach.add(new_ft)
    tab_cach.end()
    table_dest.commit()



